using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using System.Diagnostics;
using System.Reflection;

namespace LocalizeMe
{
    /// <summary>
    /// Attribute used to have localized descriptions of properties
    /// </summary>
    [AttributeUsage(AttributeTargets.Property)]
    public sealed class LocalDescriptionAttribute : DescriptionAttribute
    {

        #region Variables prives

        private string _key;

        #endregion

        #region Constructors

        /// <summary>
        /// .ctor
        /// </summary>
        /// <param name="key">The key of the dictionnary corresponding to the item to display</param>
        public LocalDescriptionAttribute(string key)
        {
            
            _key = key;

        }
        
        #endregion

        #region Proprits

        /// <summary>
        /// Returns the translated description according to the given key.
        /// </summary>
        public override string Description
        {
            get
            {
                return Resource.GetString(_key);
            }
        }
        
        /// <summary>
        /// Key of the dictionnary. Implement to be compliant with FXCop rules
        /// </summary>
        public string Key
        {
            get
            {
                return _key;
            }
        }

        #endregion

    }
}
