using System;
using System.Collections.Generic;
using System.Text;
using System.Collections.ObjectModel;
using System.Windows.Forms;
using System.IO;

#if DEBUG
using log4net;
using log4net.Config;
#endif

namespace LocalizeMe
{
    public static class Resource
    {
        private static Dictionary<string, ResourceSource> _sources = new Dictionary<string, ResourceSource>();
        private static Dictionary<string,ResourceItem> _items= new Dictionary<string,ResourceItem>();

#if DEBUG
        private static readonly ILog log = LogManager.GetLogger(typeof(LocalizeMe.Resource));
#endif

        private static string _currentCulture = "en";

        public static void AddSource(ResourceSource source)
        {
#if DEBUG
            log.Info("Entering AddSource");
            log.Debug("Adding source" + source.Name);
#endif            
            if (source != null)
            {
                if (!_sources.ContainsKey(source.Name))
                {
                    _sources.Add(source.Name, source);
                    LoadSource(source);
                    
                }
                    
#if DEBUG
                log.Debug("Source " + source.Name +" added.");
#endif
            }
            
#if DEBUG
            log.Info("Exiting AddSource");
#endif
        }

        internal static void LoadSource(ResourceSource source)
        {

#if DEBUG
            log.Info("Entering LoadSource");
#endif

            source.Load();

            ResourceItemCollection sourceItems = source.GetValues(_currentCulture);
            
            RefreshItems();
            #if DEBUG
            log.Info("Exiting LoadSource");
#endif
        }

        private static void RefreshItems()
        {
#if DEBUG
            log.Info("Entering RefreshItems");
#endif
            
            _items.Clear();
#if DEBUG
            log.Debug("Items cleared");
            log.Debug("Start loop foreach RessourceSource rs in _sources.Values");
#endif
            foreach (ResourceSource rs in _sources.Values)
            {

#if DEBUG
                log.DebugFormat("rs : " + rs.Name);

                log.Debug("Getting Values from " + rs.Name);
                log.Debug("_currentCulture : " + _currentCulture);
#endif
                ResourceItemCollection col = rs.GetValues(_currentCulture);
                if (col != null)
                {
#if DEBUG
                    log.Debug("Start loop ResourceItem ri in col");
#endif
                    foreach (ResourceItem ri in col)
                    {
#if DEBUG
                        log.Debug("Adding ri as -> ri.Key : " + ri.Key  + " ri.Value : " + ri.Value);
#endif
                        try
                        {
                            _items.Add(ri.Key, ri);
#if DEBUG
                            log.Debug("item " + ri.Key+ " added successfully");
#endif
                        }
                        catch (Exception ex)
                        {
#if DEBUG
                            log.Debug("Could not add item "+ ri.Key);
                            log.Error(ex.Message, ex);
#endif
                        }
                    }
#if DEBUG
                    log.Debug("End looping ResourceItem ri in col");
#endif
                }
                else
                {
#if DEBUG
                    log.Error("No values were found in rs " + rs.Name);
#endif
                }
            }
#if DEBUG
            log.Debug("End looping loop foreach RessourceSource rs in _sources.Values");
            log.Info("Exiting RefreshItems");
#endif

        }

        public static void SetCulture(string newculture)
        {

#if DEBUG
            log.Info("Entering SetCulture");
            log.Info("newformat : "+ newculture);

#endif

            _currentCulture = newculture;
            
            RefreshItems();
#if DEBUG
            log.Info("Exiting SetCulture");
#endif
                
        }

        public static void Save()
        {
#if DEBUG
            log.Info("Entering Save");

            log.Debug("Start loop foreach (ResourceSource rs in _sources.Values)");
#endif
            foreach (ResourceSource rs in _sources.Values)
            {
                
                try
                {
#if DEBUG
                    log.Debug("Saving "+ rs.Name);
#endif
                    rs.Save();
#if DEBUG
                    log.Debug(rs.Name  + " saved");
#endif
                }
                catch (Exception ex)
                {
#if DEBUG
                    log.Error("Error while saving "+ rs.Name);
#endif
                }
            }
#if DEBUG
            log.Debug("End loop foreach (ResourceSource rs in _sources.Values)");

            log.Info("Exiting Save");
#endif
        }

        public static void CancelChanges()
        {
#if DEBUG
            log.Info("Entering CancelChanges");

            log.Debug("Start loop foreach (ResourceItem it in _items.Values)");
#endif
            foreach (ResourceItem it in _items.Values)
            {
#if DEBUG
                log.Debug("Canceling it. it.Key -> "+ it.Key +" it.Value -> " + it.Value);
#endif
                it.CancelChanges();
            }
#if DEBUG
            log.Debug("End loop foreach (ResourceItem it in _items.Values)");
            log.Info("Exiting CancelChanges");
#endif
        }

        public static Collection<string> GetAvailableCultures()
        {
#if DEBUG
            log.Info("Entering GetAvailableCultures");
#endif
            
            Collection<string> languages = new Collection<string>();

#if DEBUG
            log.Debug("Start loop foreach (ResourceSource rs in _sources.Values)");
#endif
            foreach (ResourceSource rs in _sources.Values)
            {

#if DEBUG
                log.Debug("rs.Name : "+ rs.Name);
#endif
                Collection<string> cultures = rs.GetAvailableCultures();

                if (cultures != null)
                {
#if DEBUG
                    log.Debug("Start loop foreach (string culture in cultures)");
#endif
                    foreach (string culture in cultures)
                    {
#if DEBUG
                        log.Debug("culture: "+ culture);
#endif
                        if (!languages.Contains(culture))
                        {
#if DEBUG
                            log.Debug("Adding culture : "+ culture);
#endif
                            languages.Add(culture);
                        }
                    }
#if DEBUG
                    log.Debug("End loop foreach (string culture in cultures)");
#endif
                }
                else
                {
#if DEBUG
                    log.Error("Error getting " + rs.Name + " availaible cultures");
#endif
                }
            }
#if DEBUG
            log.Debug("End loop foreach (ResourceSource rs in _sources.Values)");
#endif
            
            return languages;
        }

        #region GetString

        /// <summary>
        /// Returns the value found in the dictionnary according to the key.
        /// </summary>
        /// <param name="key">The key in the dictionnary</param>
        /// <returns>The value if the key is found. Empty string otherwise</returns>
        public static string GetString(string key)
        {
#if DEBUG
            log.Info("Entering GetString(string key)");
            log.Debug("key : "+ key);
#endif

            if (_items == null)
            {
#if DEBUG
                log.Error("No items in the repository");
                log.Info("Exiting GetString(string key)");
#endif
                return string.Empty;
            }

            if (_items.ContainsKey(key))
            {
#if DEBUG
                log.Info("Exiting GetString(string key)");
#endif
                return _items[key].Value.ToString();
            }
            else
            {
#if DEBUG
                log.Warn("Item which key is : " + key  + " was not found in the repository");
                log.Info("Exiting GetString(string key)");
#endif
                return string.Empty;
            }
        }

        /// <summary>
        /// Returns the value found in the dictionnary according to the key.
        /// </summary>
        /// <param name="key">The key in the dictionnary</param>
        /// <param name="parameter1">The value of parameter 1 that will be replaced</param>
        /// <returns>The value if the key is found. Empty string otherwise</returns>
        public static string GetString(string key, string parameter1)
        {
#if DEBUG
            log.Info("Entering GetString(string key, string parameter1)");
#endif
            
            string str = GetString(key);
            if (string.IsNullOrEmpty(str))
                return string.Empty;
#if DEBUG
            log.Info("Exiting GetString(string key, string parameter1)");
#endif
            return str.Replace("{1}", parameter1);
        }

        /// <summary>
        /// Returns the value found in the dictionnary according to the key.
        /// </summary>
        /// <param name="key">The key in the dictionnary</param>
        /// <param name="parameter1">The value of parameter 1 that will be replaced</param>
        /// <param name="parameter2">The value of parameter 2 that will be replaced</param>
        /// <returns>The value if the key is found. Empty string otherwise</returns>
        public static string GetString(string key, string parameter1, string parameter2)
        {
#if DEBUG
            log.Info("Entering GetString(string key, string parameter1, string parameter2)");
#endif
            string str = GetString(key, parameter1);
            if (string.IsNullOrEmpty(str))
                return string.Empty;
#if DEBUG
            log.Info("Exiting GetString(string key, string parameter1, string parameter2)");
#endif
            return str.Replace("{2}", parameter2);
        }

        /// <summary>
        /// Returns the value found in the dictionnary according to the key.
        /// </summary>
        /// <param name="key">The key in the dictionnary</param>
        /// <param name="parameter1">The value of parameter 1 that will be replaced</param>
        /// <param name="parameter2">The value of parameter 2 that will be replaced</param>
        /// <param name="parameter3">The value of parameter 3 that will be replaced</param>
        /// <returns>The value if the key is found. Empty string otherwise</returns>
        public static string GetString(string key, string parameter1, string parameter2, string parameter3)
        {
#if DEBUG
            log.Info("Entering GetString(string key, string parameter1, string parameter2, string parameter3)");
#endif
            string str = GetString(key, parameter1, parameter2);
            if (string.IsNullOrEmpty(str))
                return string.Empty;
#if DEBUG
            log.Info("Exiting GetString(string key, string parameter1, string parameter2, string parameter3)");
#endif
            return str.Replace("{3}", parameter3);
        }

        #endregion

        #region Chargements des contrles et formulaires

        public static void LoadForm(Form form)
        {
            //LoadForm(form, string.Empty);
#if DEBUG
            log.Info("Entering LoadForm");
#endif

            if (form == null)
            {
#if DEBUG
                log.Error("form is null");
                log.Info("Exiting LoadForm");
#endif
                return;
            }

            string name=form.GetType().FullName;
#if DEBUG
            log.Debug("Form.Name : " + name);
#endif
            form.Text = GetString(name);

#if DEBUG
            log.Debug("Start loop foreach (Control ctrl in form.Controls)");
#endif
            foreach (Control ctrl in form.Controls)
            {
                LoadControl(name, ctrl);
            }
#if DEBUG
            log.Debug("End loop foreach (Control ctrl in form.Controls)");
            log.Info("Exiting LoadForm");
#endif
            
        }

        /// <summary>
        /// Loads all the translations for a control
        /// </summary>
        /// <param name="ctrl">The control to parse</param>
        /// <param name="frm">The form on which the control is.</param>
        public static void LoadControl(string name, Control ctrl)
        {
#if DEBUG
            log.Info("Entering LoadControl");
#endif

            if (_items.ContainsKey(name + "." + ctrl.Name))
            {
#if DEBUG
                log.Debug("Sets form name value");
#endif
                ctrl.Text = GetString(name + "." + ctrl.Name);
            }

            ToolStrip ts = ctrl as ToolStrip;

            if (ts != null)
            {
#if DEBUG
                log.Debug("Control is a ToolString");
                log.Debug("Start loop foreach (ToolStripItem item in ts.Items)");
#endif
                foreach (ToolStripItem item in ts.Items)
                {
#if DEBUG
                    log.Debug("item.Name : "+ item.Name);
#endif

                    ToolStripMenuItem tsmi = item as ToolStripMenuItem;
                    if (tsmi != null)
                    {
#if DEBUG
                        log.Debug("item is a ToolStripMenuItem");
#endif
                        LoadMenuItem(name, tsmi);
                    }
                    else
                    {
#if DEBUG
                        log.Debug("item is a not ToolStripMenuItem");
#endif
                        item.Text = GetString(name + "." + item.Name);
                    }
                }
#if DEBUG
                log.Debug("End loop foreach (ToolStripItem item in ts.Items)");
                log.Info("Exiting LoadControl");
#endif
                return;
            }

            TabControl tab=ctrl as TabControl;

            if (tab != null)
            {
#if DEBUG
                log.Debug("Control is a TabControl");
                log.Debug("Start loop foreach (TabPage tp in tab.TabPages)");
#endif
                foreach (TabPage tp in tab.TabPages)
                {
                    tp.Text = GetString(name + "." + tab.Name + "." + tp.Name);
                }
#if DEBUG
                log.Debug("End loop foreach (TabPage tp in tab.TabPages)");
#endif
            }

#if DEBUG
            log.Debug("Start loop foreach (Control c in ctrl.Controls)");
#endif

            TreeView tvw = ctrl as TreeView;

            if (tvw != null)
            {
#if DEBUG
                log.Debug("Control is a Treeview");
                log.Debug("Start loop foreach (TreeNode node in tvw.Nodes)");
#endif
                foreach (TreeNode node in tvw.Nodes)
                {
                    LoadNode(node, name + "." + tvw.Name);
                    
                }
#if DEBUG
                log.Debug("End loop foreach (TreeNode node in tvw.Nodes)");
#endif
            }

             foreach (Control c in ctrl.Controls)
                LoadControl(name, c);
#if DEBUG
            log.Debug("End loop foreach (Control c in ctrl.Controls)");
            log.Info("Exiting LoadControl");
#endif
        }

        /// <summary>
        /// Loads all the translations for a menu item
        /// </summary>
        /// <param name="frm">The form on which the control is.</param>
        /// <param name="item">The menu item to parse.</param>
        private static void LoadMenuItem(string name, ToolStripMenuItem item)
        {
#if DEBUG
            log.Info("Entering LoadMenuItem");

            log.Debug("item.Name "+ item.Name);
#endif

            if (!string.IsNullOrEmpty(GetString(name + "." + item.Name)))
            {
#if DEBUG
                log.Debug("Assigning text");
#endif
                item.Text = GetString(name + "." + item.Name);
            }

#if DEBUG
            log.Debug("Start loop  for (int i = 0; i < item.DropDownItems.Count; i++)");
#endif
            for (int i = 0; i < item.DropDownItems.Count; i++)
            {
                if (!(item.DropDownItems[i] is ToolStripSeparator))
                {
#if DEBUG
                    log.Debug("item is a ToolStripSeparator");
#endif
                    LoadMenuItem(name, (item.DropDownItems[i] as ToolStripMenuItem));
                }
            }
#if DEBUG
            log.Debug("End loop  for (int i = 0; i < item.DropDownItems.Count; i++)");


            log.Info("Exiting LoadMenuItem");
#endif
        }

        private static void LoadNode(TreeNode node,string baseName)
        {
            node.Text = GetString(baseName + "." + node.Name);
            if (node.Nodes != null && node.Nodes.Count > 0)
            {
                foreach (TreeNode tmpNode in node.Nodes)
                    LoadNode(tmpNode, baseName);
            }
        }

        #endregion
    }
}
